function [newWorldOrientation, newWorldLocation, ScoreFinal, MaskYfinal, MaskZfinal] = optCalPar(R0, t0, maskX, Kmat, pts3D)
% Function that carries out the optimization of the objective function and 
% gives in output the optimized extrinsic parameters.

% INPUT:
% R0: initial guess of the rotation matrix.
% t0: initial guess of the translation vector.
% maskX: map of values generated starting from the object contourn. 
% Kmat: intrinsic matrix.
% pts3D: points of the point cloud.

% OUTPUT:
% newWorldOrientation: optimized world orientation.
% newWorldLocation: optimized world location.
% ScoreFinal: final score obtained with the optimization.
% MaskYFinal: final binary image in which each the pixel has value 1 if at 
%             least one 3D point is projected onto that pixel, 0 otherwise.
% 
% MaskZfinal: final maskZ.



eul0 = rotm2eul(R0);

% Objective function to minimize
fun = @(ext_par) objFunction(ext_par,pts3D,maskX,Kmat);

% Setting upper and lower bounds of the search space
dim1 = pi/9 + 0.1;
dim2 = 21;
UB1 = eul0(1) + dim1;
LB1 = eul0(1) - dim1;
UB2 = eul0(2) + dim1;
LB2 = eul0(2) - dim1;
UB3 = eul0(3) + dim1;
LB3 = eul0(3) - dim1;
UB4 = t0(1) + dim2;
LB4 = t0(1) - dim2;
UB5 = t0(2) + dim2;
LB5 = t0(2) - dim2;
UB6 = t0(3) + dim2;
LB6 = t0(3) - dim2;
UB = [UB1, UB2, UB3, UB4, UB5, UB6];
LB = [LB1, LB2, LB3, LB4, LB5, LB6];


% Creating an optimization problem 
opts = optimoptions(@fmincon,'Algorithm','interior-point','PlotFcn',@optimplotfval);
problem = createOptimProblem('fmincon','x0',[eul0, t0],'objective',fun,'lb',LB,'ub',UB,'options',opts);

% Running the optimization 
gs = GlobalSearch('NumTrialPoints',4000, 'NumStageOnePoints',300); 
[ext_par,~] = run(gs,problem);

% Refinement using fminsearch
options = optimset('PlotFcns',@optimplotfval);
X0 = ext_par;
ext_par = fminsearch(fun,X0,options); 

% Computing others optimal quantities starting from the optimal extrinsic
% parameter ext_par obtained
newR = eul2rotm(ext_par(1:3));
newt = ext_par(4:6);
newWorldOrientation = newR';
newWorldLocation = - newt*newR';
[ScoreFinal, MaskYfinal, MaskZfinal] = objFunction(ext_par,pts3D,maskX,Kmat);