function [] = show3DTcloud(vetT, ptCloud, mapColor, rangeCmap)
% Function that displays the point cloud with the temperatures added, with 
% a specific color map. The points to which no temperature is associated 
% are displayed with the same color.

% INPUT:
% vetT:         temperature vector associated with the point cloud.
% ptCloud:      point cloud.
% mapColor:     colormap to apply to the temperature vector, that
%               identifies the colors of the point cloud. 
% rangeCmap:    range of the colormap.


N = ptCloud.Count;

% Counting the non-NaN values
count = 0;
for i = 1:N
    if ~isnan(vetT(i))
        count = count + 1;
    end
end

% Constructing a new vector without the NaN values
vetT_full = zeros(count,1);
count = 1;
for i = 1:N
    if ~isnan(vetT(i))
        vetT_full(count) = vetT(i);
        count = count + 1;
    end
end

% Defining the colormap range
if isempty(rangeCmap) 
    T1 = min(vetT_full);
    T2 = max(vetT_full);
else
    T1 = rangeCmap(1);
    T2 = rangeCmap(2);
end
crange = [T1 T2];

% Associating a colormap to the temperature vector
rgb = vals2colormap(vetT_full, mapColor, crange);  

% Color of the points with no temperature associated (RGB code)
color_no_T = [171 205 239];  % or [255 228 196] 

% Constructing a color matrix to be assigned to the point cloud object
count = 1;
ColorMatrix = uint8(zeros(N,3));
for i = 1:N
    if isnan(vetT(i)) 
        ColorMatrix(i,:) = color_no_T;
    else
        ColorMatrix(i,:) = 255*rgb(count,:);
        count = count + 1;
    end
end
ptCloud.Color = uint8(ColorMatrix);

% Computing some temperature values for the colorbar 
h = 0:0.25:1;
Tcolorbar = 1/10*round(10*(h*(T2 - T1) + T1));

% Visualized dimensions of the points of the cloud 
M_size = 50; 


figure
pcshow(ptCloud,'MarkerSize',M_size); 
xlabel('X [mm]');
ylabel('Y [mm]');
zlabel('Z [mm]');

% Adding colorbar and changing the style
c1 = colorbar('Ticks', h,'TickLabels',{string(Tcolorbar(1)), string(Tcolorbar(2)), string(Tcolorbar(3)), string(Tcolorbar(4)), string(Tcolorbar(5))});
c1.Location = 'southoutside';
c1.Color = 'y';
c1.Label.String = 'Temperature [C]';
c1.Label.Color = 'y';
colormap(c1, mapColor);
set(gca,'fontsize',10);
set(gca,'FontWeight','bold');
set(gca,'LineWidth',1.5);
set(gca,'TickLength',[0.018, 0.018]);


end