function [vetT, Theta, Conf, worldLocation, worldOrientation] = thermography3D(pts2D, pts3D, ptCloud, MatT, mode, dispFig)
% Function that carries out the integration between the point cloud and the
% thermogram.

% INPUT
% pts2D e pts3D:    homologous 2D and 3D points.
% ptCloud:          point cloud.
% MatT:             matrix temperature.
% mode:             type of extrinsic calibration: 0: by homologous point
%                   correspondance; 1: automatic, by the silhouette method.
% dispFig:          if 1, displays some figures.

% OUTPUT
% vetT:             temperature vector associated with the point cloud.
% Theta:            viewing angle vector associated with the point cloud.
% Conf:             confidence vector associated with the point cloud.
% worldLocation:    location of the optical centre of the thermal camera
%                   with respect to the world reference frame.
% worldOrientation: rotation matrix which describes the orientation of the 
%                   camera reference frame with respect to the world 
%                   reference frame.


N = ptCloud.Count;

I = mat2gray(MatT); 

% Intrinsic parameters for the two different resolution of the thermal
% camera used
[Rv, Rh] = size(MatT);
switch Rv
    case 240
        Res = 1;
    case 480 
        Res = 2;
    otherwise
        Res = 0;
end
[cameraParams,Kmat] = camPar(Res);

% Estimation of the pose of the camera from the homologous points
[worldOrientation, worldLocation] = estimateWorldCameraPose(pts2D, pts3D, cameraParams);

R = worldOrientation';  % Rotation Matrix
t = - worldLocation*R ; % Translation Vector

camDir = worldOrientation(3,:); % Vector normal to the camera screen
upVect = - worldOrientation(2,:);
otherAxis = worldOrientation(1,:);

ProjectedPoints = worldToImage(cameraParams, R, t, pts3D);

% Mean reprojection error
MRE_ = mre(pts2D, ProjectedPoints);
if mode == 0
    fprintf('MRE = %3.2f\n',MRE_);
end

figure
imshow(I);

% Figure showing the projection of the 3D points onto the 2D points
if dispFig == 1
    figure
    imshow(I); 
    hold on
    plot(pts2D(:,1), pts2D(:,2), 'mo', ProjectedPoints(:,1), ProjectedPoints(:,2), 'y+');
    axis square
    axis equal
    xlim([0 Rh]);
    ylim([0 Rv]);
    lg = legend('2D Points', '3D Reprojected Points');
    lg.Color = 'k';
    lg.TextColor = 'w';
    lg.Location = 'best';
    ticks = [0 0.25 0.5 0.75 1];
    maxT = max(max(MatT));
    minT = min(min(MatT));
    dif = maxT - minT;
    c1 = colorbar('Ticks', ticks,'TickLabels',{string(round(minT)), string(round(minT + 0.25*dif)), string(round(minT + 0.5*dif)), string(round(minT + 0.75*dif)), string(round(maxT))});
    c1.Label.String = 'Temperature [C]';
    %caxis([0 300])
    set(gca,'fontsize',12);
    set(gca,'FontWeight','bold');
    set(gca,'LineWidth',1.5);
    set(gca,'TickLength',[0.018, 0.018]);
%-------------------------------------------------------------------------%

% Figure showing the pose of the camera with respect to the world
    figure
    pcshow(ptCloud);
    xlabel('X');
    ylabel('Y');
    zlabel('Z');
    title('Point cloud and thermal camera pose');
    hold on
    plotCamera('Location', worldLocation, 'Orientation', worldOrientation, 'Size', 20);
    quiver3(worldLocation(1), worldLocation(2), worldLocation(3), camDir(1), camDir(2), camDir(3), 100,'m');
    quiver3(worldLocation(1), worldLocation(2), worldLocation(3), upVect(1), upVect(2), upVect(3), 100,'g');
    quiver3(worldLocation(1), worldLocation(2), worldLocation(3), otherAxis(1), otherAxis(2), otherAxis(3), 100, 'b');
    plot3(worldLocation(1), worldLocation(2), worldLocation(3),'co');
%-------------------------------------------------------------------------%
end

%--------------------- AUTOMATIC EXTRINSIC CALIBRATION -------------------%
if mode == 1
    % Extraction of the object silhouette from the thermogram
    maskX = fmaskX(MatT);

    % Creating a map starting from the silhouette 
    [~,maskX2] = createMap(maskX, MatT);

    % Perturbed euler angles and translation parameters (to test the algorithm)
    eul_R_p = rotm2eul(R) + [-pi/9 0 pi/9];
    t_p = t + [20 -20 20];
    
    % Situation before the application of the algorithm
    [Score, maskY, ~] = objFunction([eul_R_p, t_p],ptCloud.Location,maskX2,Kmat);
    fprintf('Initial Score: %4.2f\n',-Score);
    displayMasks(maskX,maskY);
    title('Before');

    % Calibration via the algorithm
    [newWO, newWL, Score_f, maskYfinal, ~] = optCalPar(eul2rotm(eul_R_p), t_p, maskX2, Kmat, ptCloud.Location);
    fprintf('Final Score: %4.2f\n',-Score_f);
    
    % Rotation matrix and translation vector
    newR = newWO';
    newt = - newWL*newR;
    
    % Comparison between the two calibration methods
    fprintf('\nEuler angles in degrees NORMAL CAL: %3.2f %3.2f %3.2f\n', rotm2eul(R)*180/pi);
    fprintf('Euler angles in degrees AUTO CAL: %3.2f %3.2f %3.2f\n', rotm2eul(newR)*180/pi);
    fprintf('Translation vector in mm NORMAL CAL: %4.2f %4.2f %4.2f\n', t);
    fprintf('Translation vector in in mm AUTO CAL: %4.2f %4.2f %4.2f\n\n', newt);
    fprintf('Euler angles difference in degrees: %3.3f %3.3f %3.3f\n', (rotm2eul(R)-rotm2eul(newR))*180/pi);
    fprintf('Translation vector difference in mm: %3.3f %3.3f %3.3f\n\n', t - newt);
    %fprintf('WorldLoc NORMAL CAL %3.2f %3.2f %3.2f\n',worldLocation);
    %fprintf('WorldLoc AUTO CAL %3.2f %3.2f %3.2f\n', newWL);
    %fprintf('DELTA WorldLoc in mm: %3.3f %3.3f %3.3f\n\n', worldLocation - newWL);
    %fprintf('RELATIVE DELTA (percentage) translation vector: %3.2f %3.2f %3.2f\n\n', abs(100*(t - newt)./t));
    
    displayMasks(maskX,maskYfinal);
    title('After');
end
%-------------------------------------------------------------------------%

if mode == 1
    AllProjPoints = worldToImage(cameraParams, newR, newt, ptCloud.Location);
else
    AllProjPoints = worldToImage(cameraParams, R, t, ptCloud.Location); 
end


%%% Inizializzazione Variabili %%%
dist = NaN(N,1,'single');
Theta = NaN(N,1,'single'); 
PixelsPoints = NaN(N,2); 

PointsToCam = single(worldLocation - ptCloud.Location);


%---------- Association of the temperatures T to the pixels --------------%
for i = 1:N
    dist(i) = norm(PointsToCam(i,:));
     if ((AllProjPoints(i,1) >= Rh + 0.5 || AllProjPoints(i,1) < 0.5) || (AllProjPoints(i,2) >= Rv + 0.5 || AllProjPoints(i,2) < 0.5))   
        PixelsPoints(i,:) = [NaN, NaN];
    else
        PixelsPoints(i,1) = round(AllProjPoints(i,2));
        PixelsPoints(i,2) = round(AllProjPoints(i,1));
    end
end


%------------------------------ Exclusion I ------------------------------%
f_r = ceil(Rv*Rh/N)+1;                          % Parameter "csi"  
VetP = round(PixelsPoints/f_r + 1);
Rnew = round([Rv, Rh]/f_r + 1);
chi = 5/(1-f_r/20);                             % Parameter "chi"
Ind = excludePoints(VetP, dist, Rnew, chi);
for i = 1:N
    if ~isnan(Ind(i))
        PixelsPoints(Ind(i),:) = [NaN NaN];
    end
end


% --------------------- Exclusion II and computing Theta -----------------%
for i = 1:N
    if ~isnan(PixelsPoints(i,1))
        ProdScal = dot(PointsToCam(i,:), single(ptCloud.Normal(i,:)));
        Theta(i) = acos(ProdScal/dist(i));
        if ProdScal < 0
            PixelsPoints(i,:) = [NaN NaN];
        end
    end
end


%---------------------- Creation of the T vector -------------------------%
vetT = NaN(N,1);
MatT2 = erodeIm(MatT);   
for i = 1:N
    if ~isnan(PixelsPoints(i,1)) 
        vetT(i) = MatT2(PixelsPoints(i,1), PixelsPoints(i,2));  
    end
end


%---------------- Association of a confidence Conf to each T -------------%
kappa = 2;  % Overlapping parameter
Conf = NaN(N,1,'single');
for i = 1:N
    if ~isnan(vetT(i))
        Conf(i) = exp(-kappa*Theta(i));
    end
end


end
